@@ -13,8 +13,9 @@ import pickle
 import os
 from pkg_resources import iter_entry_points
 from functools import partial
-from collections import defaultdict
+from collections import defaultdict, OrderedDict
 import matplotlib as mpl
+from psyplot.compat.pycompat import get_default_value
 from psyplot_gui import rcParams
 from threading import Thread
 import logging
@@ -28,12 +29,14 @@ if backend is not None:
 
 from psyplot_gui.console import ConsoleWidget
 from psyplot_gui.compat.qtcompat import (
-    QMainWindow, QApplication, Qt, QMenu, QAction, QDesktopWidget,
-    QFileDialog, QKeySequence, QtCore, with_qt5, QMessageBox, QIcon)
+    QMainWindow, QApplication, Qt, QMenu, QAction, QDesktopWidget, QLabel,
+    QFileDialog, QKeySequence, QtCore, with_qt5, QMessageBox, QIcon,
+    QInputDialog)
 from psyplot_gui.content_widget import (
     ProjectContentWidget, DatasetTree, FiguresTree)
 from psyplot_gui.plot_creator import PlotCreator
 from psyplot_gui.help_explorer import HelpExplorer
+from psyplot_gui.dataframeeditor import DataFrameEditor
 from psyplot_gui.fmt_widget import FormatoptionWidget
 from psyplot_gui.common import PyErrorMessage, get_icon, StreamToLogger
 from psyplot_gui.preferences import (
@@ -44,10 +47,12 @@ from psyplot.docstring import docstrings
 import psyplot.plotter as psyp
 import psyplot.project as psy
 import psyplot
+import psyplot.data as psyd
 import psyplot_gui
+import xarray as xr
 
 
-#: The :class:`PyQt4.QtGui.QMainWindow` of the graphical user interface
+#: The :class:`PyQt5.QtWidgets.QMainWindow` of the graphical user interface
 mainwindow = None
 
 
@@ -62,6 +67,9 @@ class MainWindow(QMainWindow):
     #: open_files_server
     open_external = QtCore.pyqtSignal(list)
 
+    #: The server to open external files
+    open_files_server = None
+
     #: Inprocess console
     console = None
 
@@ -80,9 +88,26 @@ class MainWindow(QMainWindow):
     #: help explorer
     help_explorer = None
 
+    #: the DataFrameEditor widgets, a widget to show and edit data frames
+    dataframeeditors = None
+
     #: tab widget displaying the arrays in current main and sub project
     project_content = None
 
+    #: The dockwidgets of this instance
+    dockwidgets = []
+
+    #: default widths of the dock widgets
+    default_widths = {}
+
+    _is_open = False
+
+    #: The keyboard shortcuts of the default layout
+    default_shortcuts = []
+
+    #: The current keyboard shortcuts
+    current_shortcuts = []
+
     @property
     def logger(self):
         """The logger of this instance"""
@@ -112,11 +137,16 @@ class MainWindow(QMainWindow):
             QMainWindow.AnimatedDocks | QMainWindow.AllowNestedDocks |
             QMainWindow.AllowTabbedDocks)
         #: Inprocess console
-        self.console = ConsoleWidget(parent=self)
+        self.console = ConsoleWidget(self)
         self.project_actions = {}
 
         self.config_pages = []
 
+        self.open_file_options = OrderedDict([
+            ('new psyplot plot from dataset', self.open_external_files),
+            ('new psyplot project', partial(self.open_external_files, [])),
+            ])
+
         # ---------------------------------------------------------------------
         # ----------------------------- Menus ---------------------------------
         # ---------------------------------------------------------------------
@@ -130,7 +160,7 @@ class MainWindow(QMainWindow):
         self.new_plot_action.setStatusTip(
             'Use an existing dataset (or open a new one) to create one or '
             'more plots')
-        self.new_plot_action.setShortcut(QKeySequence.New)
+        self.register_shortcut(self.new_plot_action, QKeySequence.New)
         self.new_plot_action.triggered.connect(lambda: self.new_plots(True))
         self.file_menu.addAction(self.new_plot_action)
 
@@ -140,14 +170,16 @@ class MainWindow(QMainWindow):
         self.file_menu.addMenu(self.open_project_menu)
 
         self.open_mp_action = QAction('New main project', self)
-        self.open_mp_action.setShortcut(QKeySequence.Open)
+        self.register_shortcut(self.open_mp_action, QKeySequence.Open)
         self.open_mp_action.setStatusTip('Open a new main project')
         self.open_mp_action.triggered.connect(self.open_mp)
         self.open_project_menu.addAction(self.open_mp_action)
 
         self.open_sp_action = QAction('Add to current', self)
-        self.open_sp_action.setShortcut(QKeySequence(
-            'Ctrl+Shift+O', QKeySequence.NativeText))
+
+        self.register_shortcut(
+            self.open_sp_action, QKeySequence(
+                'Ctrl+Shift+O', QKeySequence.NativeText))
         self.open_sp_action.setStatusTip(
             'Load a project as a sub project and add it to the current main '
             'project')
@@ -156,17 +188,17 @@ class MainWindow(QMainWindow):
 
         # ----------------------- Save project --------------------------------
 
-        self.save_project_menu = QMenu('Save project', parent=self)
+        self.save_project_menu = QMenu('Save', parent=self)
         self.file_menu.addMenu(self.save_project_menu)
 
-        self.save_mp_action = QAction('All', self)
+        self.save_mp_action = QAction('Full psyplot project', self)
         self.save_mp_action.setStatusTip(
             'Save the entire project into a pickle file')
-        self.save_mp_action.setShortcut(QKeySequence.Save)
+        self.register_shortcut(self.save_mp_action, QKeySequence.Save)
         self.save_mp_action.triggered.connect(self.save_mp)
         self.save_project_menu.addAction(self.save_mp_action)
 
-        self.save_sp_action = QAction('Selected', self)
+        self.save_sp_action = QAction('Selected psyplot project', self)
         self.save_sp_action.setStatusTip(
             'Save the selected sub project into a pickle file')
         self.save_sp_action.triggered.connect(self.save_sp)
@@ -174,22 +206,23 @@ class MainWindow(QMainWindow):
 
         # ------------------------ Save project as ----------------------------
 
-        self.save_project_as_menu = QMenu('Save project as', parent=self)
+        self.save_project_as_menu = QMenu('Save as', parent=self)
         self.file_menu.addMenu(self.save_project_as_menu)
 
-        self.save_mp_as_action = QAction('All', self)
+        self.save_mp_as_action = QAction('Full psyplot project', self)
         self.save_mp_as_action.setStatusTip(
             'Save the entire project into a pickle file')
-        self.save_mp_as_action.setShortcut(QKeySequence.SaveAs)
+        self.register_shortcut(self.save_mp_as_action,
+                                       QKeySequence.SaveAs)
         self.save_mp_as_action.triggered.connect(
-            partial(self.save_mp, new_name=True))
+            partial(self.save_mp, new_fname=True))
         self.save_project_as_menu.addAction(self.save_mp_as_action)
 
-        self.save_sp_as_action = QAction('Selected', self)
+        self.save_sp_as_action = QAction('Selected psyplot project', self)
         self.save_sp_as_action.setStatusTip(
             'Save the selected sub project into a pickle file')
         self.save_sp_as_action.triggered.connect(
-            partial(self.save_sp, new_name=True))
+            partial(self.save_sp, new_fname=True))
         self.save_project_as_menu.addAction(self.save_sp_as_action)
 
         # -------------------------- Pack project -----------------------------
@@ -197,13 +230,13 @@ class MainWindow(QMainWindow):
         self.pack_project_menu = QMenu('Zip project files', parent=self)
         self.file_menu.addMenu(self.pack_project_menu)
 
-        self.pack_mp_action = QAction('All', self)
+        self.pack_mp_action = QAction('Full psyplot project', self)
         self.pack_mp_action.setStatusTip(
             'Pack all the data of the main project into one folder')
         self.pack_mp_action.triggered.connect(partial(self.save_mp, pack=True))
         self.pack_project_menu.addAction(self.pack_mp_action)
 
-        self.pack_sp_action = QAction('Selected', self)
+        self.pack_sp_action = QAction('Selected psyplot project', self)
         self.pack_sp_action.setStatusTip(
             'Pack all the data of the current sub project into one folder')
         self.pack_sp_action.triggered.connect(partial(self.save_sp, pack=True))
@@ -214,19 +247,21 @@ class MainWindow(QMainWindow):
         self.export_project_menu = QMenu('Export figures', parent=self)
         self.file_menu.addMenu(self.export_project_menu)
 
-        self.export_mp_action = QAction('All', self)
+        self.export_mp_action = QAction('Full psyplot project', self)
         self.export_mp_action.setStatusTip(
             'Pack all the data of the main project into one folder')
         self.export_mp_action.triggered.connect(self.export_mp)
-        self.export_mp_action.setShortcut(QKeySequence(
-            'Ctrl+E', QKeySequence.NativeText))
+        self.register_shortcut(
+            self.export_mp_action, QKeySequence(
+                'Ctrl+E', QKeySequence.NativeText))
         self.export_project_menu.addAction(self.export_mp_action)
 
-        self.export_sp_action = QAction('Selected', self)
+        self.export_sp_action = QAction('Selected psyplot project', self)
         self.export_sp_action.setStatusTip(
             'Pack all the data of the current sub project into one folder')
-        self.export_sp_action.setShortcut(QKeySequence(
-            'Ctrl+Shift+E', QKeySequence.NativeText))
+        self.register_shortcut(
+            self.export_sp_action, QKeySequence(
+                'Ctrl+Shift+E', QKeySequence.NativeText))
         self.export_sp_action.triggered.connect(self.export_sp)
         self.export_project_menu.addAction(self.export_sp_action)
 
@@ -237,9 +272,10 @@ class MainWindow(QMainWindow):
         self.close_project_menu = QMenu('Close project', parent=self)
         self.file_menu.addMenu(self.close_project_menu)
 
-        self.close_mp_action = QAction('Main project', self)
-        self.close_mp_action.setShortcut(QKeySequence(
-            'Ctrl+Shift+W', QKeySequence.NativeText))
+        self.close_mp_action = QAction('Full psyplot project', self)
+        self.register_shortcut(
+            self.close_mp_action, QKeySequence(
+                'Ctrl+Shift+W', QKeySequence.NativeText))
         self.close_mp_action.setStatusTip(
             'Close the main project and delete all data and plots out of '
             'memory')
@@ -247,11 +283,11 @@ class MainWindow(QMainWindow):
             lambda: psy.close(psy.gcp(True).num))
         self.close_project_menu.addAction(self.close_mp_action)
 
-        self.close_sp_action = QAction('Only selected', self)
+        self.close_sp_action = QAction('Selected psyplot project', self)
         self.close_sp_action.setStatusTip(
             'Close the selected arrays project and delete all data and plots '
             'out of memory')
-        self.close_sp_action.setShortcut(QKeySequence.Close)
+        self.register_shortcut(self.close_sp_action, QKeySequence.Close)
         self.close_sp_action.triggered.connect(
             lambda: psy.gcp().close(True, True))
         self.close_project_menu.addAction(self.close_sp_action)
@@ -260,9 +296,11 @@ class MainWindow(QMainWindow):
 
         if sys.platform != 'darwin':  # mac os makes this anyway
             self.quit_action = QAction('Quit', self)
+            self.quit_action.triggered.connect(self.close)
             self.quit_action.triggered.connect(
                 QtCore.QCoreApplication.instance().quit)
-            self.quit_action.setShortcut(QKeySequence.Quit)
+            self.register_shortcut(
+                self.quit_action, QKeySequence.Quit)
             self.file_menu.addAction(self.quit_action)
 
         self.menuBar().addMenu(self.file_menu)
@@ -287,7 +325,8 @@ class MainWindow(QMainWindow):
 
         self.help_action = QAction('Preferences', self)
         self.help_action.triggered.connect(lambda: self.edit_preferences(True))
-        self.help_action.setShortcut(QKeySequence.Preferences)
+        self.register_shortcut(self.help_action,
+                                       QKeySequence.Preferences)
         self.help_menu.addAction(self.help_action)
 
         # ---------------------------- About ----------------------------------
@@ -303,61 +342,59 @@ class MainWindow(QMainWindow):
             lambda: self.show_dependencies(True))
         self.help_menu.addAction(self.dependencies_action)
 
+        self.dockwidgets = []
+
         # ---------------------------------------------------------------------
         # -------------------------- Dock windows -----------------------------
         # ---------------------------------------------------------------------
         #: tab widget displaying the arrays in current main and sub project
-        self.project_content = ProjectContentWidget(parent=self)
-        self.project_content.to_dock(self, 'Plot objects',
-                                     Qt.LeftDockWidgetArea)
         #: tree widget displaying the open datasets
+        self.project_content = ProjectContentWidget(parent=self)
         self.ds_tree = DatasetTree(parent=self)
-        self.ds_tree.to_dock(self, 'Datasets', Qt.LeftDockWidgetArea)
         #: tree widget displaying the open figures
         self.figures_tree = FiguresTree(parent=self)
-        self.figures_tree.to_dock(self, 'Figures', Qt.LeftDockWidgetArea)
         #: help explorer
         self.help_explorer = help_explorer = HelpExplorer(parent=self)
-        help_explorer.to_dock(self, 'Help explorer', Qt.RightDockWidgetArea)
+        if help_explorer.viewers['HTML help'].sphinx_thread is not None:
+            help_explorer.viewers[
+                'HTML help'].sphinx_thread.html_ready.connect(
+                    self.focus_on_console)
+        #: the DataFrameEditor widgets
+        self.dataframeeditors = []
         #: general formatoptions widget
         self.fmt_widget = FormatoptionWidget(
             parent=self, help_explorer=help_explorer,
-            shell=self.console.kernel_client.kernel.shell)
-        self.fmt_widget.to_dock(self, 'Formatoptions', Qt.BottomDockWidgetArea)
+            console=self.console)
 
         # load plugin widgets
-        self.plugins = plugins = {}
-        logger = self.logger
-        inc = rcParams['plugins.include']
-        exc = rcParams['plugins.exclude']
-        for ep in iter_entry_points('psyplot_gui'):
-            plugin_name = '%s:%s:%s' % (ep.module_name, ':'.join(ep.attrs),
-                                        ep.name)
-            # check if the user wants to explicitly this plugin
-            include_user = None
-            if inc:
-                include_user = (
-                    ep.module_name in inc or ep.name in inc or
-                    '%s:%s' % (ep.module_name, ':'.join(ep.attrs)) in inc)
-            if include_user is None and exc == 'all':
-                include_user = False
-            elif include_user is None:
-                # check for exclude
-                include_user = not (
-                    ep.module_name in exc or ep.name in exc or
-                    '%s:%s' % (ep.module_name, ':'.join(ep.attrs)) in exc)
-            if not include_user:
-                logger.debug('Skipping plugin %s: Excluded by user',
-                             plugin_name)
-                continue
-            logger.debug('Loading plugin %s', plugin_name)
-            w_class = ep.load()
-            plugins[plugin_name] = w = w_class(parent=self)
-            w.to_dock(self)
+        self.plugins = plugins = OrderedDict([
+            ('project_content', self.project_content),
+            ('ds_tree', self.ds_tree),
+            ('figures_tree', self.figures_tree),
+            ('help_explorer', self.help_explorer),
+            ('fmt_widget', self.fmt_widget),
+            ])
+        self.default_plugins = list(plugins)
+        for plugin_name, w_class in six.iteritems(rcParams.load_plugins()):
+            plugins[plugin_name] = w_class(parent=self)
 
-        self.windows_menu.addSeparator()
         self.add_mp_to_menu()
         psy.Project.oncpchange.connect(self.eventually_add_mp_to_menu)
+        self.windows_menu.addSeparator()
+        self.window_layouts_menu = QMenu('Window layouts', self)
+        self.restore_layout_action = QAction('Restore default layout', self)
+        self.restore_layout_action.triggered.connect(self.setup_default_layout)
+        self.window_layouts_menu.addAction(self.restore_layout_action)
+        self.windows_menu.addMenu(self.window_layouts_menu)
+        self.panes_menu = QMenu('Panes', self)
+        self.windows_menu.addMenu(self.panes_menu)
+
+        self.dataframe_menu = QMenu('DataFrame editors', self)
+        self.dataframe_menu.addAction(
+            'New Editor', partial(self.new_data_frame_editor, None,
+                                  'DataFrame Editor'))
+        self.dataframe_menu.addSeparator()
+        self.windows_menu.addMenu(self.dataframe_menu)
 
         # ---------------------------------------------------------------------
         # -------------------------- connections ------------------------------
@@ -385,6 +422,15 @@ class MainWindow(QMainWindow):
         if show:
             self.help_explorer.show_intro(self.console.intro_msg)
 
+        # ---------------------------------------------------------------------
+        # ------------------------- open_files_server -------------------------
+        # ---------------------------------------------------------------------
+        self.callbacks = {'new_plot': self.open_external.emit,
+                          'change_cwd': self._change_cwd,
+                          'run_script': self.console.run_script.emit,
+                          'command': self.console.run_command.emit,
+                          }
+
         # Server to open external files on a single instance
         self.open_files_server = socket.socket(socket.AF_INET,
                                                socket.SOCK_STREAM,
@@ -400,15 +446,93 @@ class MainWindow(QMainWindow):
         self.config_pages.extend([GuiRcParamsWidget, PsyRcParamsWidget])
 
         # display the statusBar
-        self.statusBar()
+        statusbar = self.statusBar()
+        self.figures_label = QLabel()
+        statusbar.addWidget(self.figures_label)
+        self.plugin_label = QLabel()
+        statusbar.addWidget(self.plugin_label)
+
+        self.default_widths = {}
+
+        self.setup_default_layout()
+
         if show:
             self.showMaximized()
 
-        # hide plugin widgets that should be hidden at startup
+        # save the default widths after they have been shown
         for w in self.plugins.values():
+            self.default_widths[w] = w.dock.size().width()
+
+        # hide plugin widgets that should be hidden at startup. Although this
+        # has been executed by :meth:`setup_default_layout`, we have to execute
+        # it again after the call of showMaximized
+        for w in self.plugins.values():
+            w.to_dock(self)
             if w.hidden:
                 w.hide_plugin()
 
+        self._is_open = True
+
+    def focus_on_console(self, *args, **kwargs):
+        """Put focus on the ipython console"""
+        self.console._control.setFocus()
+
+    def new_data_frame_editor(self, df=None, title='DataFrame Editor'):
+        """Open a new dataframe editor
+
+        Parameters
+        ----------
+        df: pandas.DataFrame
+            The dataframe to display
+        title: str
+            The title of the dock window
+
+        Returns
+        -------
+        psyplot_gui.dataframeeditor.DataFrameEditor
+            The newly created editor"""
+        editor = DataFrameEditor()
+        self.dataframeeditors.append(editor)
+        editor.to_dock(self, title,
+                       Qt.RightDockWidgetArea, docktype='df')
+        if df is not None:
+            editor.set_df(df)
+        editor.show_plugin()
+        editor.maybe_tabify()
+        editor.raise_()
+        return editor
+
+    def setup_default_layout(self):
+        """Set up the default window layout"""
+        self.project_content.to_dock(self, 'Plot objects',
+                                     Qt.LeftDockWidgetArea)
+        self.ds_tree.to_dock(self, 'Datasets', Qt.LeftDockWidgetArea)
+        self.figures_tree.to_dock(self, 'Figures', Qt.LeftDockWidgetArea)
+        self.help_explorer.to_dock(self, 'Help explorer',
+                                   Qt.RightDockWidgetArea)
+        self.fmt_widget.to_dock(self, 'Formatoptions', Qt.BottomDockWidgetArea)
+
+        modify_widths = bool(self.default_widths)
+        for w in map(self.plugins.__getitem__, self.default_plugins):
+            w.show_plugin()
+
+            if modify_widths and with_qt5:
+                self.resizeDocks([w.dock], [self.default_widths[w]],
+                                 Qt.Horizontal)
+
+        # hide plugin widgets that should be hidden at startup
+        for w in self.plugins.values():
+            w.to_dock(self)
+            if w.hidden:
+                w.hide_plugin()
+
+        action2shortcut = defaultdict(list)
+        for s, a in self.default_shortcuts:
+            action2shortcut[a].append(s)
+
+        for a, s in action2shortcut.items():
+            self.register_shortcut(a, s)
+
     def _save_project(self, p, new_fname=False, *args, **kwargs):
         if new_fname or 'project_file' not in p.attrs:
             fname = QFileDialog.getSaveFileName(
@@ -424,7 +548,7 @@ class MainWindow(QMainWindow):
             fname = p.attrs['project_file']
         try:
             p.save_project(fname, *args, **kwargs)
-        except:
+        except Exception:
             self.error_msg.showTraceback('<b>Could not save the project!</b>')
         else:
             p.attrs['project_file'] = fname
@@ -441,7 +565,7 @@ class MainWindow(QMainWindow):
 
     def _open_project(self, *args, **kwargs):
         fname = QFileDialog.getOpenFileName(
-            self, 'Project destination', os.getcwd(),
+            self, 'Project file', os.getcwd(),
             'Pickle files (*.pkl);;'
             'All files (*)'
             )
@@ -478,7 +602,7 @@ class MainWindow(QMainWindow):
             return
         try:
             p.export(fname, *args, **kwargs)
-        except:
+        except Exception:
             self.error_msg.showTraceback(
                 '<b>Could not export the figures!</b>')
 
@@ -495,8 +619,7 @@ class MainWindow(QMainWindow):
             except RuntimeError:
                 pass
         self.plot_creator = PlotCreator(
-            self.console.get_obj, help_explorer=self.help_explorer,
-            parent=self)
+            help_explorer=self.help_explorer, parent=self)
         available_width = QDesktopWidget().availableGeometry().width() / 3.
         width = self.plot_creator.sizeHint().width()
         height = self.plot_creator.sizeHint().height()
@@ -505,6 +628,10 @@ class MainWindow(QMainWindow):
         if exec_:
             self.plot_creator.exec_()
 
+    def excepthook(self, type, value, traceback):
+        """A method to replace the sys.excepthook"""
+        self.error_msg.excepthook(type, value, traceback)
+
     def edit_preferences(self, exec_=None):
         """Edit Spyder preferences"""
         if hasattr(self, 'preferences'):
@@ -612,7 +739,9 @@ class MainWindow(QMainWindow):
         ret = super(MainWindow, self).addDockWidget(area, dockwidget, *args,
                                                     **kwargs)
         if docktype == 'pane':
-            self.windows_menu.addAction(dockwidget.toggleViewAction())
+            self.panes_menu.addAction(dockwidget.toggleViewAction())
+        elif docktype == 'df':
+            self.dataframe_menu.addAction(dockwidget.toggleViewAction())
         return ret
 
     def start_open_files_server(self):
@@ -625,7 +754,7 @@ class MainWindow(QMainWindow):
         port = rcParams['main.open_files_port']
         try:
             self.open_files_server.bind(('127.0.0.1', port))
-        except:
+        except Exception:
             return
         self.open_files_server.listen(20)
         while 1:  # 1 is faster than True
@@ -638,60 +767,115 @@ class MainWindow(QMainWindow):
                 # To avoid a traceback after closing on Windows
                 if e.args[0] == eintr:
                     continue
+                # handle a connection abort on close error
+                enotsock = (errno.WSAENOTSOCK if os.name == 'nt'
+                            else errno.ENOTSOCK)
+                if e.args[0] in [errno.ECONNABORTED, enotsock]:
+                    return
                 raise
-            l = pickle.loads(req.recv(1024))
-            self.open_external.emit(l)
+            args = pickle.loads(req.recv(1024))
+            callback = args[0]
+            func = self.callbacks[callback]
+            self.logger.debug('Emitting %s callback %s', callback, func)
+            func(args[1:])
             req.sendall(b' ')
 
+    def change_cwd(self, path):
+        """Change the current working directory"""
+        import os
+        os.chdir(path)
+
+    def _change_cwd(self, args):
+        path = args[0][0]
+        self.change_cwd(path)
+
     docstrings.keep_params(
         'make_plot.parameters', 'fnames', 'project', 'engine', 'plot_method',
-        'name', 'dims', 'encoding', 'enable_post')
+        'name', 'dims', 'encoding', 'enable_post', 'seaborn_style',
+        'concat_dim', 'chname')
+
+    def open_files(self, fnames):
+        """Open a file and ask the user how"""
+        fnames_s = ', '.join(map(os.path.basename, fnames))
+        if len(fnames_s) > 30:
+            fnames_s = fnames_s[:27] + '...'
+        item, ok = QInputDialog.getItem(
+            self, 'Open file...', 'Open %s as...' % fnames_s,
+            list(self.open_file_options), current=0, editable=False)
+        if ok:
+            return self.open_file_options[item](fnames)
 
     @docstrings.get_sectionsf('MainWindow.open_external_files')
     @docstrings.dedent
     def open_external_files(self, fnames=[], project=None, engine=None,
                             plot_method=None, name=None, dims=None,
-                            encoding=None, enable_post=False):
+                            encoding=None, enable_post=False,
+                            seaborn_style=None, concat_dim=get_default_value(
+                                xr.open_mfdataset, 'concat_dim'), chname={}):
         """
         Open external files
 
         Parameters
         ----------
-        %(make_plot.parameters.fnames|project|engine|plot_method|name|dims|encoding|enable_post)s
+        %(make_plot.parameters.fnames|project|engine|plot_method|name|dims|encoding|enable_post|seaborn_style|concat_dim|chname)s
         """
+        if seaborn_style is not None:
+            import seaborn as sns
+            sns.set_style(seaborn_style)
         if project is not None:
             fnames = [s.split(',') for s in fnames]
+            if not isinstance(project, dict):
+                project = psyd.safe_list(project)[0]
             single_files = (l[0] for l in fnames if len(l) == 1)
             alternative_paths = defaultdict(lambda: next(single_files, None))
             alternative_paths.update(list(l for l in fnames if len(l) == 2))
             p = psy.Project.load_project(
                 project, alternative_paths=alternative_paths,
                 engine=engine, main=not psy.gcp(), encoding=encoding,
-                enable_post=enable_post)
+                enable_post=enable_post, chname=chname)
             if isinstance(project, six.string_types):
                 p.attrs.setdefault('project_file', project)
+            return True
         else:
             self.new_plots(False)
-            self.plot_creator.open_dataset(fnames, engine=engine)
-            self.plot_creator.insert_array(name)
+            self.plot_creator.open_dataset(fnames, engine=engine,
+                                           concat_dim=concat_dim)
+            if name == 'all':
+                ds = self.plot_creator.get_ds()
+                name = sorted(set(ds.variables) - set(ds.coords))
+            self.plot_creator.insert_array(
+                list(filter(None, psy.safe_list(name))))
             if dims is not None:
+                ds = self.plot_creator.get_ds()
+                dims = {key: ', '.join(
+                    map(str, val)) for key, val in six.iteritems(
+                        dims)}
+                for i, vname in enumerate(
+                        self.plot_creator.array_table.vnames):
+                    self.plot_creator.array_table.selectRow(i)
+                    self.plot_creator.array_table.update_selected(
+                        )
                 self.plot_creator.array_table.selectAll()
+                var = ds[vname[0]]
                 self.plot_creator.array_table.update_selected(
-                    dims={key: ', '.join(
-                        map(str, val)) for key, val in six.iteritems(
-                            dims)})
+                    dims=var.psy.decoder.correct_dims(var, dims.copy()))
             if plot_method:
-                self.plot_creator.pm_combo.setCurrentText(plot_method)
+                self.plot_creator.pm_combo.setCurrentIndex(
+                    self.plot_creator.pm_combo.findText(plot_method))
             self.plot_creator.exec_()
+            return True
 
-    def _open_external_files(self, l):
-        self.open_external_files(*l)
+    def _open_external_files(self, args):
+        self.open_external_files(*args)
 
     @classmethod
     @docstrings.get_sectionsf('MainWindow.run')
     @docstrings.dedent
     def run(cls, fnames=[], project=None, engine=None, plot_method=None,
-            name=None, dims=None, encoding=None, enable_post=False, show=True):
+            name=None, dims=None, encoding=None, enable_post=False,
+            seaborn_style=None,
+            concat_dim=get_default_value(xr.open_mfdataset, 'concat_dim'),
+            chname={}, show=True):
         """
         Create a mainwindow and open the given files or project
 
@@ -718,10 +902,33 @@ class MainWindow(QMainWindow):
         if fnames or project:
             mainwindow.open_external_files(
                 fnames, project, engine, plot_method, name, dims, encoding,
-                enable_post)
+                enable_post, seaborn_style, concat_dim, chname)
         psyplot.with_gui = True
         return mainwindow
 
+    def register_shortcut(self, action, shortcut,
+                          context=Qt.ApplicationShortcut):
+        """Register an action for a shortcut"""
+        shortcuts = psy.safe_list(shortcut)
+        for j, shortcut in enumerate(shortcuts):
+            found = False
+            for i, (s, a) in enumerate(self.current_shortcuts):
+                if s == shortcut:
+                    new_shortcuts = [
+                        sc for sc in self.current_shortcuts[i][1].shortcuts()
+                        if sc != s]
+                    a.setShortcut(QKeySequence())
+                    if new_shortcuts:
+                        a.setShortcuts(new_shortcuts)
+                    self.current_shortcuts[i][1] = action
+                    found = True
+                    break
+            if not found:
+                self.default_shortcuts.append([shortcut, action])
+                self.current_shortcuts.append([shortcut, action])
+        action.setShortcuts(shortcuts)
+        action.setShortcutContext(context)
+
     @classmethod
     @docstrings.dedent
     def run_app(cls, *args, **kwargs):
@@ -741,7 +948,18 @@ class MainWindow(QMainWindow):
         cls.run(*args, **kwargs)
         sys.exit(app.exec_())
 
-    def close(self, *args, **kwargs):
+    def closeEvent(self, event):
+        """closeEvent reimplementation"""
+        if not self._is_open or (self._is_open and self.close()):
+            self._is_open = False
+            event.accept()
+
+    def close(self):
         _set_mainwindow(None)
-        self.open_files_server.close()
-        super(MainWindow, self).close(*args, **kwargs)
+        if self.open_files_server is not None:
+            self.open_files_server.close()
+            del self.open_files_server
+        for widget in self.plugins.values():
+            widget.close()
+        self.plugins.clear()
+        return super(MainWindow, self).close()
